﻿local ALPHA = "ALPHA"
local UNKNOWN = "UNKNOWN"
local releaseString = nil
LootCouncil = LibStub("AceAddon-3.0"):NewAddon("LootCouncil", "AceEvent-3.0", "AceConsole-3.0", "AceComm-3.0", "AceTimer-3.0", "AceHook-3.0")
local addon = LootCouncil
local L = LibStub("AceLocale-3.0"):GetLocale("LootCouncil")
local df = LibStub("LibDeformat-3.0")
local dbicon = LibStub("LibDBIcon-1.0")
local db, dbpc
local pname = UnitName("player")
local windowString = ""
local timer, timercount, ttscantimer
local settings = {}
local is_standby = false
LootCouncil.settings = settings
local releaseRevision = nil
do -- Borrowed from BigWigs :)
	-- START: MAGIC WOWACE VOODOO VERSION STUFF
	local releaseType = RELEASE
	
	
	--@alpha@
	-- The following code will only be present in alpha ZIPs.
	releaseType = ALPHA
	--@end-alpha@

	-- This will (in ZIPs), be replaced by the highest revision number in the source tree.
	releaseRevision = tonumber("30")

	-- If the releaseRevision ends up NOT being a number, it means we're running a SVN copy.
	if type(releaseRevision) ~= "number" then
		releaseRevision = -1
	end

	-- Then build the release string, which we can add to the interface option panel.
	local majorVersion = GetAddOnMetadata("LootCouncil", "Version") or "0.?"
	if releaseRevision == -1 then
		releaseString = L["You are running a source checkout of LootCouncil %s directly from the repository."]:format(majorVersion) -- Version message, %s = majorVersion
		windowString = "(Development)"
	elseif releaseType == RELEASE then
		windowString = "(" .. majorVersion .. "Release)"
	elseif releaseType == ALPHA then
		releaseString = L["You are running an ALPHA RELEASE of LootCouncil %s (revision %d). Please report any bugs @ http://www.wowace.com/addons/LootCouncil/tickets/"]:format(majorVersion, releaseRevision) -- Version message, %s = majorVersion, %d = revision
		windowString = "(" .. majorVersion .. "r" .. releaseRevision .. " ALPHA)"
	end
	_G.LOOTCOUNCIL_RELEASE_TYPE = releaseType
	_G.LOOTCOUNCIL_RELEASE_REVISION = releaseRevision
	_G.LOOTCOUNCIL_RELEASE_STRING = releaseString
	-- END:   MAGIC WOWACE VOODOO VERSION STUFF
end

local ignoredItems = {
	[30316] = true, -- Devastation
	[30317] = true, -- Cosmic Infuser
	[30312] = true, -- Infinity Blade
	[30318] = true, -- Netherstrand Longbow
	[30313] = true, -- Staff of Disintegration
	[30311] = true, -- Warp Slicer
	[30320] = true, -- Bundle of Nether Spikes
	[30314] = true, -- Phaseshift Bulwark
	[30319] = true, -- Nether Spikes
	[29434] = true, -- Badge of Justice
	[40752] = true, -- Emblem of Heroism
	[40753] = true, -- Emblem of Valor
	[45624] = true, -- Emblem of Conquest
	[47241] = true, -- Emblem of Triumph
	[49426] = true, -- Emblem of Frost
	[22450] = true, -- Void Crystal
	[20725] = true, -- Nexus Crystal
	[34057] = true, -- Abyss Crystal
}

local objectLocations = {
	[L["Trial of the Crusader"]] = true,
	[L["Deathbringer's Rise"]] = true,
	[L["The Frostwing Halls"]] = true,
	[L["The Eventide"]] = true,
	[L["Rampart of Skulls"]] = true,
}
local lootObjects = {
	[L["Gunship Armory"]] = true,
	[L["Deathbringer's Cache"]] = L["Deathbringer Saurfang"],
	[L["Cache of the Dreamwalker"]] = L["Valithria Dreamwalker"],
	[L["Alexstrasza's Gift"]] = true,
	[L["Argent Crusade Tribute Chest"]] = true,
	[L["Champions' Cache"]] = true,
	[L["Cache of Innovation"]] = true,
	[L["Cache of Living Stone"]] = true,
	[L["Cache of Storms"]] = true,
	[L["Cache of Winter"]] = true,
	[L["Freya's Gift"]] = true,
	[L["Gift of the Observer"]] = true,
	[L["Rare Cache of Winter"]] = true,
	[L["The Arsenal Absolute"]] = true
	-- [L["Scourge Transporter"]] = true
}
local function MeSpew(text, ...)
	--[===[@debug@
	if Spew then
		Spew(tostring(text), ...)
	end
	--@end-debug@]===]
end

local function count(t)
	local c = 0
	for k in pairs(t) do
		c = c +1
	end
	return c
end
local function split(str, pat)
   local t = {}  -- NOTE: use {n = 0} in Lua-5.0
   local fpat = "(.-)" .. pat
   local last_end = 1
   local s, e, cap = str:find(fpat, 1)
   while s do
      if s ~= 1 or cap ~= "" then
	 table.insert(t,cap)
      end
      last_end = e+1
      s, e, cap = str:find(fpat, last_end)
   end
   if last_end <= #str then
      cap = str:sub(last_end)
      table.insert(t, cap)
   end
   return t
end

-- Parse options. Options are inside GuildInfo and are inside a -EPGP-
-- block. Possible options are:
--
-- bid_time = 30,
-- vote_time = 60,
-- rarity = 0,
-- roll_percent = .50,
-- roll_out_of = 1000,
-- channels = {"Raid"},
-- win_mode = percent, majority
local global_config_defs = {
  bid_time = {
    pattern = "@bid_time:(%d+)",
    parser = tonumber,
    validator = function(v) return v >= 0 and v <= 100 end,
    error = L["Bid Time should be a number between 0 and 100"],
    default = 30,
    -- change_message = "DecayPercentChanged",
  },
  vote_time = {
    pattern = "@vote_time:(%d+)",
    parser = tonumber,
    validator = function(v) return v >= 0 and v <= 100 end,
    error = L["Vote Time should be a number between 0 and 100"],
    default = 60,
    -- change_message = "ExtrasPercentChanged",
  },
  rarity = {
    pattern = "@rarity:(%d+)",
    parser = tonumber,
    validator = function(v) return v >= 0 and v <= 6 end,
    error = L["Rarity should be a number between 0 and 6"],
    default = 4,
    -- change_message = "MinEPChanged",
  },
  roll_percent = {
    pattern = "@roll_percent:(%d+)",
    parser = tonumber,
    validator = function(v) return v >= 0 and v <= 100 end,
    error = L["Roll Percent should be a number between 0 and 100"],
    default = 50,
    -- change_message = "BaseGPChanged",
  },
  roll_out_of = {
    pattern = "@roll_out_of:(%d+)",
    parser = tonumber,
    validator = function(v) return v >= 0 end,
    error = L["Roll Out Of should be a positive number"],
    default = 100,
    -- change_message = "BaseGPChanged",
  },
  roll_time_out = {
    pattern = "@roll_time_out:(%d+)",
    parser = tonumber,
    validator = function(v) return v >= 0 end,
    error = L["Roll Time Out should be a positive number"],
    default = 30,
    -- change_message = "BaseGPChanged",
  },
  private = {
    pattern = "@private:(.+)$",
    parser = function(v)
		return strtrim(v)
	end,
    validator = function(v) return (v == "false" or v == "true") end,
    error = L["Roll Time Out should 'true' or 'false'"],
    default = "false",
    -- change_message = "BaseGPChanged",
  },
  channels = {
    pattern = "@channels:(.+)",
    parser = function(v) 
		return split(v, ', ?')
	end,
    validator = function(v) return #v >= 1 end,
    error = L["Channels should be a list of channel names"],
    default = {"Raid"},
    -- change_message = "BaseGPChanged",
  },
  announce = {
	pattern = "@announce:(.+)",
    parser = function(v) 
		return split(v, ', ?')
	end,
    validator = function(v) return #v >= 1 end,
    error = L["Announce should be a list of channel names"],
    default = {"Raid Warning"},
  },
}
-- Set defaults
for var, def in pairs(global_config_defs) do
  settings[var] = def.default
end
local function ParseGuildInfo(info)
  
  local lines = {string.split("\n", info)}
  local in_block = false

  local new_config = {}

  for _,line in pairs(lines) do
    if line == "-LootCouncil-" then
      in_block = not in_block
    elseif in_block then
      for var, def in pairs(global_config_defs) do
        local v = line:match(def.pattern)
        if v then
          -- print(string.format("Matched [%s]", line))
          v = def.parser(v)
          if v == nil or not def.validator(v) then
            LootCouncil:Print(def.error)
          else
            new_config[var] = v
          end
        end
      end
    end
  end

  for var, def in pairs(global_config_defs) do
    local new_value = new_config[var]
    if new_value == nil then
      new_value = def.default
    end
    if settings[var] ~= new_value then
      -- print(string.format("Setting new value %s for variable %s",
            -- tostring(new_value), var))
      settings[var] = new_value
      --callbacks:Fire(def.change_message, new_value)
    end
  end
  addon:Print("Updated Settings")
end

function addon:OnInitialize()
	addon.loots = {}
	addon.mobs = {}
	addon:SetEnabledState(false)
	--- Setup Databases, Options
	addon.firstload = true
	addon.councillors = {}
	addon.versions = {}
	
    self.db = LibStub("AceDB-3.0"):New("LootCouncilDB", {
			profile = {
				debug = false,
				closetime = 30,
				voteclosetime = 60,
				raritythreshold = 0,
				rollpercent = .50,
				rolloutof = 1000,
				icon = {},
				optionmaster = pname,
				channels = {"Raid"},
				ranks = {}
			},
		})
	db = self.db.profile
	local LDB = LibStub("LibDataBroker-1.1", true)
	launcher = LDB:NewDataObject("LootCouncil", {
		type = "data source",
		label = "LootCouncil",
		text = addon:IsEnabled() and "|cff00FF00Enabled|r" or "|cffFF0000Standby|r",
		icon = addon:IsEnabled() and "Interface\\ICONS\\INV_Misc_GroupNeedMore" or "Interface\\ICONS\\INV_Misc_GroupLooking",
		OnClick = function(clickedframe, button)
			if button == "RightButton" then 
				if addon:IsEnabled() then
					addon:Disable()
				else
					addon:Enable()
				end
			else 
				if IsControlKeyDown() then 
					addon:ShowConfig() 
				else 
					addon:ToggleWindow()
				end 
			end
		end,
		OnTooltipShow = function(tt) 
			tt:AddLine(L["LootCouncil"]) -- LDB Tooltip title

			
			tt:AddLine(L["|cff00FF00Click to open the loot council window|r"]) -- LDB click help: toggle
			--tt:AddLine(L["|cff00FF00Shift-click to take ontime attendance time|r"]) -- LDB right-click help: options
			--tt:AddLine(L["|cff00FF00Ctrl-click to open options|r"]) -- LDB right-click help: options
			tt:AddLine(L["|cff00FF00Right-click to enable/disable|r"]) -- LDB right-click help: options
		end,
	})
	--local shown = true
	dbicon:Register("LootCouncil", launcher, addon.db.profile.icon)
	
	-- get timezone for time(), useful for uploads into GMT/UTC
	local function get_timezone()
		local now = time()
		return difftime(now, time(date("!*t", now)))
	end
	-- Return a timezone string in ISO 8601:2000 standard form (+hhmm or -hhmm)
	local function get_tzoffset(timezone)
		local h, m = math.modf(timezone / 3600)
		return string.format("%+.4d", 100 * h + 60 * m)
	end
	db.timezone = tonumber(get_tzoffset(get_timezone()))
	addon:RegisterComm("LootCouncil")
	self:RegisterChatCommand("lootcouncil", "ToggleWindow")
	addon:CommHandlers()
	addon:RegisterEvent("RAID_ROSTER_UPDATE")
	-- addon:RegisterEvent("PARTY_MEMBERS_CHANGED", "RAID_ROSTER_UPDATE")
	
	-- if type(StaticPopupDialogs) ~= "table") then StaticPopupDialogs = {} end
	StaticPopupDialogs["CONFIRM_LOOT_DISTRIBUTION_LC"] = {
		text = CONFIRM_LOOT_DISTRIBUTION,
		button1 = YES,
		button2 = NO,
		OnAccept = function(self, data)
			-- Spew("data", data)
			
			-- print("Confirmed give loot")
			if data.number == nil then
				data.number = 1
			end
			-- print(("Confirmed give loot: %s, %s"):format(tostring(data.slot[data.number]), tostring(data.id)))
			GiveMasterLoot(data.slot[data.number], data.id);
			-- Spew("data", data)
			if data.itemID ~= nil then
				-- addon:DistributeLoot(data.itemID, data.winners, data.number)
				-- MeSpew("confirm distribute winners", data.winners)
				addon:ScheduleTimer("DistributeLoot", .1, {data.itemID, data.winners, data.number})
			end
		end,
		OnCancel = function(self, data)
			-- print(("Confirmed give loot: %s, %s"):format(data.slot, data.id))
			-- print("Canceled give loot")
			-- GiveMasterLoot(data.slot, data.id);
			-- Spew("data", data)
			if data.number == nil then
				data.number = 1
			end
			
			if data.itemID ~= nil then
				-- addon:DistributeLoot(data.itemID, data.winners, data.number)
				-- MeSpew("cancel distribute winners", data.winners)
				addon:ScheduleTimer("DistributeLoot", .1, {data.itemID, data.winners, data.number})
			end
		end,
		timeout = 0,
		hideOnEscape = 1,
		hasItemFrame = 1,
	}
	StaticPopupDialogs["LC_CLEARLOOTS"] = {
		text = L["Clear All Loot information?\nNote: This is irreversible"],
		button1 = YES,
		button2 = NO,
		OnAccept = function(self, data)
			wipe(addon.loots)
		end,
		timeout = 30,
		hideOnEscape = 1,
	}
	addon:SecureHookScript(GameTooltip, "OnShow", "Mouse")
	-- addon:RegisterEvent("CURSOR_UPDATE", "Mouse")
	-- addon:RegisterEvent("CHAT_MSG_OFFICER", "ParseChat")
	--addon:Disable()
	
	for i=1,13 do
		_G["GuildFrameGuildStatusButton" .. i]:SetScript("OnEnter", addon.PlayerTooltipOnEnter)
		_G["GuildFrameGuildStatusButton" .. i]:SetScript("OnLeave", addon.PlayerTooltipOnLeave)
		_G["GuildFrameButton" .. i]:SetScript("OnEnter", addon.PlayerTooltipOnEnter)
		_G["GuildFrameButton" .. i]:SetScript("OnLeave", addon.PlayerTooltipOnLeave)
		-- self:HookScript(getglobal("GuildFrameGuildStatusButton" .. i), "OnEnter", "PlayerTooltipOnEnter")
		-- self:HookScript(getglobal("GuildFrameGuildStatusButton" .. i), "OnLeave", "PlayerTooltipOnLeave")
	end
	addon:RegisterEvent("GUILD_ROSTER_UPDATE")
	-- addon:GUILD_ROSTER_UPDATE()
end

function addon:Mouse(event)
	if GameTooltip:IsShown() and GameTooltip:NumLines() >= 1 and not GameTooltip:IsUnit("mouseover") then
		local name = GetMouseFocus():GetName()
		if name == "WorldFrame" then
			local text = GameTooltipTextLeft1:GetText()
			for object, bossname in pairs(lootObjects) do
				if object == text then
					if type(bossname) == "string" then
						addon.lastObject = bossname
					else
						addon.lastObject = text
					end
					MeSpew(event or "timer", addon.lastObject)
					break
				end
			end
			-- Spew(event or "timer", )
		end
	end
end

function addon:GameTooltipOnShow(frame)
	MeSpew("GameTooltip", "OnShow")
end

-- function addon:LeaveParty()
	-- addon:SendComm("bye")
	-- print("LeaveParty")
-- end

function addon:ZoneChange(event)
	if addon.lastObject ~= nil then addon.lastObject = nil end
	--@non-debug@
	if select(2, IsInInstance()) == "raid" then
	--@end-non-debug@
		addon:CancelTimer(ttscantimer, true)
		for loc in pairs(objectLocations) do
			-- Spew("loc", loc)
			if (GetSubZoneText() ~= "" and loc == GetSubZoneText()) or (GetRealZoneText() == loc) then
				ttscantimer = self:ScheduleRepeatingTimer("Mouse", .5)
				MeSpew(event, GetRealZoneText(), GetSubZoneText())
				break
			end
		end
	--@non-debug@
	end
	--@end-non-debug@
end

function addon:OnEnable()
	-- addon:RegisterEvent("GUILD_ROSTER_UPDATE")
	addon:GUILD_ROSTER_UPDATE()
	addon:RegisterEvent("LOOT_OPENED")
	addon:RegisterEvent("LOOT_CLOSED")
	addon:RegisterEvent("CHAT_MSG_LOOT")
	local guild_info = GetGuildInfoText()
	ParseGuildInfo(guild_info)
	addon:RegisterEvent("ZONE_CHANGED", "ZoneChange")
	addon:RegisterEvent("ZONE_CHANGED_INDOORS", "ZoneChange")
	addon:ZoneChange("ZONE_INIT")
	launcher.text = "|cff00FF00Enabled|r"
	launcher.icon = "Interface\\ICONS\\INV_Misc_GroupNeedMore" -- Tracking icon
	
	--[[if IsInGuild() then 
		wipe(db.guildinfo.ranks)
		for i = 1, GuildControlGetNumRanks() do
			db.guildinfo.ranks[i] = GuildControlGetRankName(i)
		end
		db.guildinfo.name = GetGuildInfo("player")
		db.guildinfo.server = GetRealmName()
	end]]
	
	--[[if self.firstload then
		self:Print(releaseString)
		self.firstload = false
	end]]
	--[===[@debug@
	-- [[
	-- self:AddCouncillor("Lyrotha", 9)
	-- self:AddCouncillor("Whiteycm", 10)
	-- self:AddCouncillor("Janosszabo", 9)
	-- self:AddCouncillor("Cereis")
	-- self:AddCouncillor("Egonor")
	-- self:AddCouncillor("Mopery")
	-- self:AddLoot(52026, 1)
	-- self:AddLoot("|cffa335ee|Hitem:52025:0:0:0:0:0:0:0:0|h[Vanquisher's Mark of Sanctification]|h|r", 2)
	-- self:AddMessage(52025, "Lyrotha", "\124cffa335ee\124Hitem:50828:0:0:0:0:0:0:0:0\124h[Lasherweave Raiment]\124h\124r")
	-- self:AddMessage(52025, "Flâxe", "\124cffa335ee\124Hitem:50828:0:0:0:0:0:0:0:0\124h[Lasherweave Raiment]\124h\124r")
	-- self:AddMessage(52025, "Aliysa", "\124cffa335ee\124Hitem:50828:0:0:0:0:0:0:0:0\124h[Lasherweave Raiment]\124h\124r")
	-- self:AddMessage(52025, "Cyklonex", "\124cffa335ee\124Hitem:50828:0:0:0:0:0:0:0:0\124h[Lasherweave Raiment]\124h\124r")
	-- self:AddMessage(52025, "Arvalo", "\124cffa335ee\124Hitem:50828:0:0:0:0:0:0:0:0\124h[Lasherweave Raiment]\124h\124r")
	-- ]]
	--@end-debug@]===]
end

function addon:OnDisable()
	--addon:UnregisterEvent("RAID_ROSTER_UPDATE")
	addon:UnregisterEvent("CHAT_MSG_LOOT")
	-- addon:UnregisterEvent("GUILD_ROSTER_UPDATE")
	addon:UnregisterEvent("LOOT_OPENED")
	addon:UnregisterEvent("LOOT_CLOSED")
	addon:UnregisterEvent("ZONE_CHANGED")
	addon:UnregisterEvent("ZONE_CHANGED_INDOORS")
	launcher.text = "|cffFF0000Standby|r"
	launcher.icon = "Interface\\ICONS\\INV_Misc_GroupLooking" -- Not tracking icon
end

function addon:CommHandlers()
	addon.commhandlers = {
		active = function(message, distribution, sender)
			if not addon:IsEnabled() then
				addon:Enable()
				addon:Introduce()
			end
		end,
		hello = function(sender, version)
			if not addon:IsEnabled() then
				addon:Enable()
			end
			wipe(addon.councillors)
			if not is_standby then
				addon:SendComm("hi", releaseRevision)
				addon:AddCouncillor(pname, releaseRevision)
			end
			addon:AddCouncillor(sender, tonumber(version))
		end,
		hi = function(sender, version)
			addon:AddCouncillor(sender, tonumber(version))
			addon:CheckCouncillors()
		end,
		al = function(sender, item, quantity, slot, guid, mobname)
			-- itemID
			addon:CheckCouncillors()
			-- local itemID, quantity, slot = message:match("^(.+)%%%%(.+)%%%%(.*)$")
			if item ~= nil then
				addon:AddLoot(item, tonumber(quantity), slot, sender, guid, mobname)
			else
				addon:Print("Error in 'Add Loot' Comms: find item?")
			end
		end,
		start = function(sender, containerID, itemID, length)
			-- itemID
			
			-- local itemID, opentime = message:match("^(%d+)%%%%(%d+)$")
			addon:StartLoot(containerID, tonumber(itemID), tonumber(length), sender)
		end,
		closeloot = function(sender, containerID, itemID)
			-- local itemID = tonumber(message)
			addon:CloseLoot(containerID, tonumber(itemID), sender)
		end,
		startvote = function(sender, containerID, itemID, length)
			-- local itemID, opentime = message:match("^(.+)%%%%(.+)$")
			addon:StartVote(containerID, tonumber(itemID), tonumber(length), sender)
		end,
		closevote = function(sender, containerID, itemID)
			-- local itemID = tonumber(message)
			addon:CloseVote(containerID, tonumber(itemID), sender)
		end,
		vote = function(sender, messagenum, containerID, itemID, value)
			-- player, messagenum, itemID, value
			-- local player, messagenum, itemID, value = message:match("^(.+)%%%%(.+)%%%%(.+)%%%%(.+)$")
			if tonumber(value) == 1 then value = true else value = false end
			addon:Vote(tonumber(messagenum), containerID, tonumber(itemID), value, sender)
		end,
		finishvote = function(sender, containerID, itemID)
			-- itemID
			-- local itemID = tonumber(message)
			addon:FinishVote(containerID, tonumber(itemID), sender)
		end,
		standby = function(sender)
			addon:RemoveCouncillor(sender)
		end,
		option = function(message, distribution, sender)
		
		end,
	}
end

function addon:SendComm(type, ...)
	--print(type, select("#", ...), ...)
	--if UnitInRaid("player") ~= nil then
		local message = type .. "\a"
		-- for i = 1, select("#", ...) do
			-- message = message .. tostring(select(i, ...))
			-- if i < select("#", ...) then
				-- message = message .. "%%"
			-- end
		-- end
		message = message .. table.concat({...}, "\a")
		self:SendCommMessage("LootCouncil", message, "RAID")
	--end
end

function addon:OnCommReceived(prefix, message, type, sender)
	--[===[@debug@
		print(prefix, message, type, sender)
	--@end-debug@]===]
	if prefix ~= "LootCouncil" or sender == pname then return end
	local sync, rest = strsplit("\a", message, 2)
	-- local sync, rest = select(3, message:find("(%S+)%s*(.*)$"))
	--local commtype, remainder = message:match("^(%a)(.*)")
	if (not sync) or (not addon.commhandlers[sync]) then return end
	local args = {strsplit("\a", rest)}
	-- MeSpew("args", args)
	addon.commhandlers[sync](sender, unpack(args))
end

function addon:GetContainer()
	-- get the current loot container GUID (NPC) or name (Object)
	if addon.lastObject ~= nil then
		return addon.lastObject, addon.lastObject
	elseif UnitExists("target") and not UnitIsPlayer("target") then
		return UnitGUID("target"), UnitName("target")
	else
		return L["Unknown"], L["Unknown"]
	end
end

function addon:GetItemIdFromLink(link)
	return tonumber(string.match(link, 'item:(%d+)'))
end
addon.loot_cache = {}
addon.can_auto_loot = false
function addon:LOOT_OPENED()
	numLootItems = GetNumLootItems()
	local itemIDs, length = {}, 0
	local enabled = addon:IsEnabled()
	-- Spew("enabled", enabled)
	if not enabled then
		self:Enable()
	end
	wipe(addon.loot_cache)
	for i = 1, numLootItems do
		if LootSlotIsItem(i) then
			local itemLink = GetLootSlotLink(i)
			local _, _, lootQuantity, rarity, locked = GetLootSlotInfo(i)
			if ignoredItems[self:GetItemIdFromLink(itemLink)] == nil then
				if rarity >= settings.rarity and not locked and lootQuantity > 0 then
					if addon.loot_cache[itemLink] == nil then
						addon.loot_cache[itemLink] = {}
					end
					table.insert(addon.loot_cache[itemLink], i)
					
					if itemIDs[itemLink] == nil then
						itemIDs[itemLink] = {["q"] = 1, ["s"] = select(9, GetItemInfo(itemLink)) or "None"}
					else
						itemIDs[itemLink].q = itemIDs[itemLink].q + 1
					end
					itemIDs[itemLink].guid, itemIDs[itemLink].name = addon:GetContainer()
					addon.can_auto_loot = itemIDs[itemLink].guid
					length = length + 1
				end
			end
		end
	end
	if length > 0 then
		if not enabled then
			-- self:Enable()
			if #addon.councillors <= 1 then
				addon:Introduce()
			end
			self:SendComm("active")
		end
	end
	MeSpew("itemids", itemIDs)
	for k,v in pairs(itemIDs) do
		addon:AddLoot(k, v.q, v.s, nil, v.guid, v.name)
	end
end

function addon:LOOT_CLOSED()
	addon.can_auto_loot = false
	wipe(addon.loot_cache)
end

function addon:GetMobs(first)
	local tabs, length = {}, 0
	for k in pairs(addon.loots) do
		length = length + 1
	end
	if length > 0 then
		for k,v in pairs(addon.mobs) do
			if first then
				return k
			else
				table.insert(tabs, {text=v[1], value=k})
			end
		end
		table.sort(tabs, function(a, b)
			-- MeSpew("shit", addon.mobs[a.value], addon.mobs[b.value])
			return addon.mobs[a.value][2] > addon.mobs[b.value][2]
		end)
		return tabs
	else
		return false
	end
end

function addon:GetTabs(containerID, getfirst)
	local tabs, length = {}, 0
	for k in pairs(addon.loots[containerID]) do
		length = length + 1
	end
	if length > 0 then
		for k,v in pairs(addon.loots[containerID]) do
			if not v.disabled then
				if getfirst then
					return k
				else
					local itemtext = GetItemInfo(k)
					if not itemtext then
						--table.insert(que, k)
						--addon:StartTTScan()
						itemtext = v.link:match("%[(.-)%]")
					end
					-- if v.quantity and v.quantity > 1 then
						-- itemtext = string.format("%s %sx", itemtext, v.quantity)
					-- end
					table.insert(tabs, {text=addon:GetItemText(containerID, k, "uncolored"), value=k})
				end
			end
		end
		-- table.sort(tabs, function(a, b)
			-- return addon.loots[a.value].stage < addon.loots[b.value].stage
		-- end)
		return tabs
	else
		return false
	end
end

function addon:Introduce()
	wipe(addon.councillors)
	if not is_standby then
		addon:AddCouncillor(pname, releaseRevision)
	end
	self:SendComm("hello", releaseRevision)
end

function addon:CheckCouncillors()
	if not addon:IsEnabled() then
		addon:Enable()
	end
	--print("number of councillors:", #addon.councillors)
	-- if #addon.councillors < 1 then
		-- addon:Introduce()
	-- end
end

function addon:VerifySlot(playerItem, lootItem)
	
	local player = select(9, GetItemInfo(playerItem))
	local _, lootLink, _, _, _, _, _, _, loot = GetItemInfo(lootItem)
	-- print(tostring(loot), tostring(player))
	-- print(tostring(lootLink))
	
	-- Spew("player", player)
	if player == loot then
		-- direct match, simple
		return true
	elseif addon.revslots[player] == addon.revslots[loot] then
		return true
	else
		-- more complicated
		if addon.slotinfo[self:GetItemIdFromLink(lootLink)] ~= nil then
			-- special item as defined in globals
			for _,slotnum in pairs(addon.slotinfo[self:GetItemIdFromLink(lootLink)]) do
				if type(addon.slots[slotnum]) == "string" then
					if addon.slots[slotnum] == player then
						return true
					end
				else
					for _,slot in pairs(addon.slots[slotnum]) do
						if slot == player then
							return true
						end
					end
				end
			end
		elseif (loot == "" or loot == nil) and addon.slotinfo[self:GetItemIdFromLink(lootLink)] == nil then
			return true
		end
	end
	MeSpew("things didn't match", "player", player, playerItem, "loot", loot, lootLink)
	return false
end

function addon:AddMessage(containerID, itemID, player, message)
	local loot = addon.loots[containerID][itemID]
	if addon:VerifySlot(message, loot.link) then
		if not loot.messages then
			loot.messages = {}
		end
		local update = false
		local pattern = "|cff%x*|Hitem:.-|h|r" -- Itemlink pattern
		local itemLink = string.match(message, pattern)
		for k,v in pairs(loot.messages) do
			if string.match(loot.messages[k].message, pattern) == itemLink and loot.messages[k].player == player then
				loot.messages[k].message = message
				update = true
			end
		end
		-- if not addon.loots[itemID].players[player] then
			-- addon.loots[itemID].players[player] = {}
		-- end
		if not update then
			local num = #loot.messages + 1
			loot.messages[num] = {player = player, votes = {}, message = message}
		end
		addon:RedrawTab(containerID, itemID)
	end
end

function addon:AddLoot(itemLink, quantity, slot, from, guid, mobname)
	
	local itemID = self:GetItemIdFromLink(itemLink) or itemLink
	if addon.loots[guid] == nil then
		addon.loots[guid] = {}
	end
	if addon.loots[guid][itemID] == nil then
		addon.loots[guid][itemID] = {}
	end
	if addon.mobs[guid] == nil then
		addon.mobs[guid] = {mobname, time()}
	end
	if addon.loots[guid][itemID].stage == nil or addon.loots[guid][itemID].stage < 1 then
		addon.loots[guid][itemID]["disabled"] = false
		addon.loots[guid][itemID]["stage"] = 0
		addon.loots[guid][itemID]["link"] = itemLink
		addon.loots[guid][itemID]["looted"] = {}
		addon.loots[guid][itemID]["slot"] = slot
		addon.loots[guid][itemID]["added"] = time()
		addon.loots[guid][itemID]["quantity"] = quantity or 1
		addon.loots[guid][itemID]["votedone"] = {}
	end
	if addon.window then
		addon:UpdateTabs()
	end
	if not from then
		addon:SendComm("al", itemLink, quantity, slot, guid, mobname)
	end
	return true
end

function addon:UpdateTabs()
	local currentcontainer = (addon.currentcontainer ~= nil) and addon.currentcontainer or addon:GetMobs(true)
	local currenttab = (addon.currenttab ~= nil) and addon.currenttab or addon:GetTabs(currentcontainer, true)
	if addon.window then
		addon.window.mobtabs:SetTabs(addon:GetMobs())
		addon.window.mobtabs:SelectTab(currentcontainer)
	end
end

local magicchars = {"%", "[", "]", "-", "*", "?", "+", "^", "$"}
function addon:ParseChat(event, message, sender, ...)
	if event == "CHAT_MSG_CHANNEL" then
		local channelname = select(7, ...)
		-- Spew("channelname", channelname)
		local good = false
		for k,v in pairs(settings.channels) do
			if v == channelname then
				good = true
			end
		end
		if not good then return end
	end
	--|cffa335ee|Hitem:47976:3721:3526:3526:0:0:0:849462768:80|h[Legplates of the Immortal Spider]|h|r
	--local pattern = "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?" -- Itemlink pattern
	--pattern = "|cff%x*|Hitem:%d+:?%d*:?%d*:?%d*:?%d*:?%d*:?%-?%d*:?%-?%d*:?%d*|?h?%[?[^%[%]]*%]?|h|r" -- Itemlink pattern
	local ipattern = "|cff%x*|Hitem:.-|h|r" -- Itemlink pattern
	--      |cff9d9d9d|Hitem:33553:0:  0:   0:   0:    0:   0: 0:80|h[Serrated Pincer]|h|r
	local items = {}
	for item in string.gmatch(message, ipattern) do
		-- Spew("item", item)
		--if item ~= select(2, GetItemInfo(addon.open)) then
			table.insert(items, item)
		--end
    end
	-- Spew("items", items)
	if #items > 1 then
		for k,v in pairs(items) do
			if tonumber(addon:GetItemIdFromLink(v)) ~= addon.open.itemID then
				local pattern = v
				for i=1, #magicchars do
					pattern = (pattern):gsub("%" .. magicchars[i], "%%%" .. magicchars[i])
					-- Spew("loop"..i, pattern)
				end
				local str = (message):gsub(pattern, "")
				-- str = string.gsub(str, "%s+", " ")
				str = strtrim(str)
				-- Spew("item"..k, str)
				self:AddMessage(addon.open.container, addon.open.item, sender, str)
			end
		end
	elseif #items == 1 then
		-- print(, addon.open)
		if tonumber(addon:GetItemIdFromLink(items[1])) ~= addon.open.itemID then
			self:AddMessage(addon.open.container, addon.open.item, sender, message)
		end
	end
	--[[if Spew then
		Spew("items", items)
	end]]
end

local function insertUnique(tbl, var)
	for _,v in pairs(tbl) do
		if var == v then
			return false
		end
	end
	table.insert(tbl, var)
end

function addon:AddCouncillor(name, version)
	insertUnique(self.councillors, name)
	addon.versions[name] = version
	table.sort(self.councillors, function(a, b)
		if a == pname then
			return true
		elseif b == pname then
			return false
		else
			return a < b
		end
	end)
	if addon.window then
		local str = {}
		for _,v in pairs(addon.councillors) do
			local text
			if addon.versions[v] ~= nil then
				if v ~= pname then
					if addon.versions[v] == -1 or addon.versions[v] == addon.versions[pname] then
						text = v .. " (" .. addon.versions[v] .. ")"
					elseif addon.versions[v] > addon.versions[pname] then
						text = "|cff00ff00" .. v .. " (" .. addon.versions[v] .. ")|r"
					else
						text = "|cffff0000" .. v .. " (" .. addon.versions[v] .. ")|r"
					end
				else
					text = v .. " (" .. addon.versions[v] .. ")"
				end
			else
				text = "|cffff0000" .. v .. " (??)|r"
			end
			
			table.insert(str, text)
		end
		local strstr = table.concat(str, ", ")
		addon.window.councillors:SetText(L["Councillors: %s"]:format(strstr ~= "" and strstr or L["None"]) )
	end
	addon:RedrawTab(addon.currentcontainer, self.currenttab)
end

function addon:RemoveCouncillor(name)
	local new_councillors = {}
	for k,v in pairs(self.councillors) do
		if v ~= name then
			table.insert(new_councillors, v)
		end
	end
	self.councillors = new_councillors
	addon.versions[name] = nil
	
	table.sort(self.councillors, function(a, b)
		if a == pname then
			return true
		elseif b == pname then
			return false
		else
			return a < b
		end
	end)
	if addon.window then
		local str = {}
		for _,v in pairs(addon.councillors) do
			local text
			if addon.versions[v] ~= nil then
				if v ~= pname then
					if addon.versions[v] == -1 or addon.versions[v] == addon.versions[pname] then
						text = v .. " (" .. addon.versions[v] .. ")"
					elseif addon.versions[pname] ~= nil and addon.versions[v] > addon.versions[pname] then
						text = "|cff00ff00" .. v .. " (" .. addon.versions[v] .. ")|r"
					else
						text = "|cffff0000" .. v .. " (" .. addon.versions[v] .. ")|r"
					end
				else
					text = v .. " (" .. addon.versions[v] .. ")"
				end
			else
				text = "|cffff0000" .. v .. " (??)|r"
			end
			
			table.insert(str, text)
		end
		local strstr = table.concat(str, ", ")
		addon.window.councillors:SetText(L["Councillors: %s"]:format(strstr ~= "" and strstr or L["None"]) )
	end
	addon:RedrawTab(addon.currentcontainer, self.currenttab)
end

function addon:GoStandby()
	addon:RemoveCouncillor(pname)
	addon:SendComm("standby")
end

function addon:ReturnStandby()
	addon:SendComm("hello")
end

function addon:GetItemText(containerID, item, f)
	-- print(tostring(item), tostring(f))
	local itemName, itemLink, itemID, itemCount
	if type(item) == "number" then
		-- print("I got a number", tostring(tonumber(item)))
		itemID = item
		_, itemName = string.match(addon.loots[containerID][item].link, 'item:(%d+).*%[(.-)%]')
		if itemName ~= nil then
			itemLink = addon.loots[containerID][item].link
		else
			itemName, itemLink = GetItemInfo(item)
			if itemName == nil then
				-- not in the client cache, abort
				return item
			end
		end
	else
		-- print("I got a string", tostring(item))
		itemID, itemName = string.match(item, 'item:(%d+).*%[(.-)%]')
		itemLink = item
	end
	itemID = tonumber(itemID)
	local loot = addon.loots[containerID][itemID]
	-- Spew("loots", addon.loots[itemID])
	-- print(tostring(itemID), tostring(itemName), tostring(itemLink), tostring(itemCount))
	itemCount = loot.quantity or 1
	
	if f == "full" then
		if itemCount > 1 then
			return string.format("%s %sx", itemLink, itemCount)
		else
			return itemLink
		end
	elseif f == "uncolored" then
		if itemCount > 1 then
			return string.format("%s %sx", itemName, itemCount)
		else
			return itemName
		end
	-- elseif f == "nocount" then
		-- return itemName
	elseif f == "nocountcolor" then
		return itemLink
	else
		return itemName
	end
end

function addon:Announce(text)
	for _,v in pairs(settings.announce) do
		if v:lower() == "raid" then
			if UnitInRaid("player") then
				SendChatMessage(tostring(text), "RAID")
			end
		elseif v:lower() == "raid warning" then
			if UnitInParty("player") and (IsRaidOfficer() or IsRaidLeader()) then
				SendChatMessage(tostring(text), "RAID_WARNING")
			end
		elseif v:lower() == "party" then
			if UnitInParty("player") then
				SendChatMessage(tostring(text), "PARTY")
			end
		elseif v:lower() == "guild" then
			if IsInGuild() then
				SendChatMessage(tostring(text), "GUILD")
			end
		elseif v:lower() == "officer" then
			GuildControlSetRank(select(3, GetGuildInfo("player")) + 1)
			if select(4, GuildControlGetRankFlags()) then
				SendChatMessage(tostring(text), "OFFICER")
			end
		else
			local index = GetChannelName(v)
			if (index~=nil) then 
				SendChatMessage(tostring(text), "CHANNEL", nil, index);
			end
		end
	end
end

function addon:GetCouncillors()
	return self.councillors
end

function addon:UpdateVoteCloseTime(info) -- table, 1: containerID, 2: itemID
	timercount = timercount + 1
	if timercount < addon.opentime then
		local timeleft = addon.opentime - timercount
		if info[2] == addon.currenttab and info[1] == addon.currentcontainer then
			-- local link = select(2, GetItemInfo(itemID)) and select(2, GetItemInfo(itemID)) or addon.loots[itemID].link
			local link = addon:GetItemText(info[1], info[2], "full")
			addon.window.item:SetText(link .. L[", Voting closing in %d seconds"]:format(timeleft))
		end
	else
		if timer then
			self:CancelTimer(timer, true)
		end
		addon:CloseVote(info[1], info[2], addon.loots[info[1]][info[2]].starter)
	end
end

function addon:UpdateLootTime(info) -- table, 1: containerID, 2: itemID
	timercount = timercount + 1
	if timercount < addon.opentime then
		local timeleft = addon.opentime - timercount
		if info[2] == addon.currenttab and info[1] == addon.currentcontainer then
			-- local link = select(2, GetItemInfo(itemID)) and select(2, GetItemInfo(itemID)) or addon.loots[itemID].link
			local link = addon:GetItemText(info[1], info[2], "full")
			addon.window.item:SetText(link .. L[", Closing in %d seconds"]:format(timeleft))
		end
		if timeleft <=5 and pname == addon.loots[info[1]][info[2]].starter then
			self:Announce(L["Closing in %s"]:format(timeleft))
		end
	else
		if timer then
			self:CancelTimer(timer, true)
		end
		self:CloseLoot(info[1], info[2], addon.loots[info[1]][info[2]].starter)
	end
end

function addon:StartLoot(containerID, itemID, opentime, from)
	local loot = addon.loots[containerID][itemID]
	addon.lastloot = {container = containerID, item = itemID}
	--Spew("jkfdsa", {itemID, addon.opentime})
	if addon.voting then
		if timer then
			self:CancelTimer(timer, true)
		end
		_ = addon:CloseVote(addon.voting.container, addon.voting.item, from, true)
	end
	if addon.open then
		if timer then
			self:CancelTimer(timer, true)
		end
		_ = addon:CloseLoot(addon.open.container, addon.open.item, from, true)
	end
	if not from or from == pname then 
		if itemID then
			-- local itemtext = addon.loots[itemID].link
			-- if addon.loots[itemID].quantity and addon.loots[itemID].quantity > 1 then
				-- itemtext = string.format("%s %sx", itemtext, addon.loots[itemID].quantity)
			-- end
			self:Announce(L["%s is now open for Loot Council, Closing in %s seconds. Link upgrades now to %s"]:format(addon:GetItemText(containerID, itemID, "full"), opentime or settings.bid_time, table.concat(settings.channels, L[" or "])))
		end
		addon.opentime = tonumber(settings.bid_time)
		loot.starter = pname
		addon:SendComm("start", containerID, itemID, settings.bid_time)
	else
		loot.starter = from
		addon.opentime = tonumber(opentime)
	end
	for _,v in pairs(settings.channels) do
		if v:lower() == "raid" then
			addon:RegisterEvent("CHAT_MSG_RAID", "ParseChat")
			addon:RegisterEvent("CHAT_MSG_RAID_LEADER", "ParseChat")
		elseif v:lower() == "party" then
			addon:RegisterEvent("CHAT_MSG_PARTY", "ParseChat")
			addon:RegisterEvent("CHAT_MSG_PARTY_LEADER", "ParseChat")
		elseif v:lower() == "guild" then
			addon:RegisterEvent("CHAT_MSG_GUILD", "ParseChat")
		elseif v:lower() == "officer" then
			addon:RegisterEvent("CHAT_MSG_OFFICER", "ParseChat")
		else
			addon:RegisterEvent("CHAT_MSG_CHANNEL", "ParseChat")
		end
	end
	
	timercount = 0
	if not addon.window or not addon.window:IsShown() then
		addon:ToggleWindow()
	end
	addon.open = {item = itemID, container = containerID}
	addon.window.mobtabs:SelectTab(containerID)
	if addon.open and addon.open.item == addon.currenttab and addon.open.container == addon.currentcontainer then
		
		
		-- local itemtext = select(2, GetItemInfo(itemID)) and select(2, GetItemInfo(itemID)) or addon.loots[itemID].link
		local itemtext = addon:GetItemText(containerID, itemID, "full")
		-- if addon.loots[itemID].quantity and addon.loots[itemID].quantity > 1 then
			-- itemtext = string.format("%s %sx", itemtext, addon.loots[itemID].quantity)
		-- end
		if itemtext and addon.window.item then
			--print(addon.opentime, timercount)
			addon.window.item:SetText(itemtext .. L[", Closing in %d seconds"]:format(addon.opentime - timercount))
			MeSpew("Update Text right away", itemtext)
		end
	end
	timer = self:ScheduleRepeatingTimer("UpdateLootTime", 1, {containerID, itemID})
	loot.stage = 1
	
	if not addon.window or not addon.window:IsShown() then
		addon:ToggleWindow()
		addon.window.mobtabs:SelectTab(containerID)
	end
	self:RedrawTab(containerID, itemID)
end

function addon:CloseLoot(containerID, itemID, from, novote)
	local loot = addon.loots[containerID][itemID]
	addon:UnregisterEvent("CHAT_MSG_RAID")
	addon:UnregisterEvent("CHAT_MSG_RAID_LEADER")
	addon:UnregisterEvent("CHAT_MSG_PARTY")
	addon:UnregisterEvent("CHAT_MSG_PARTY_LEADER")
	addon:UnregisterEvent("CHAT_MSG_GUILD")
	addon:UnregisterEvent("CHAT_MSG_OFFICER")
	addon:UnregisterEvent("CHAT_MSG_CHANNEL")
	
	if timer then
		self:CancelTimer(timer, true)
	end
	if not from or from == pname then
		self:Announce(L["%s is now closed for Loot Council"]:format(select(2,GetItemInfo(addon.open.item))))
	end
	if addon.open and addon.open.item == addon.currenttab and addon.open.container == addon.currentcontainer then
		-- local itemtext = select(2, GetItemInfo(itemID)) and select(2, GetItemInfo(itemID)) or addon.loots[itemID].link
		-- if addon.loots[itemID].quantity and addon.loots[itemID].quantity > 1 then
			-- itemtext = string.format("%s %sx", itemtext, addon.loots[itemID].quantity)
		-- end
		local itemtext = addon:GetItemText(containerID, itemID, "full")
		if itemtext and addon.window.item then
			addon.window.item:SetText(itemtext)
		end
	end
	addon.open = nil
	addon.opentime = nil
	if not from or from == pname then
		addon:SendComm("closeloot", containerID, itemID)
		if not novote then
			--addon.loots[itemID].stage = 0
			addon:StartVote(containerID, itemID, settings.vote_time, loot.starter)
		end
	end
	
end

function addon:StartVote(containerID, itemID, opentime, from)
	local loot = addon.loots[containerID][itemID]
	addon.lastloot = {container = containerID, item = itemID}
	if addon.voting then
		if timer then
			self:CancelTimer(timer, true)
		end
		_ = addon:CloseVote(addon.voting.container, addon.voting.item, from, true)
	end
	if addon.open then
		if timer then
			self:CancelTimer(timer, true)
		end
		_ = addon:CloseLoot(addon.open.container, addon.open.item, from, true)
	end
	if not from or from == pname then
		addon.opentime = settings.vote_time
		addon:SendComm("startvote", containerID, itemID, opentime)
	else	
		addon.opentime = opentime
	end
	if loot.messages and #loot.messages > 0 then
		addon.voting = {item = itemID, container = containerID}
		loot.stage = 2
		addon:RedrawTab(containerID, itemID)
		timercount = 0
		if not addon.window or not addon.window:IsShown() then
			addon:ToggleWindow()
		end
		addon.window.mobtabs:SelectTab(containerID)
		if addon.voting.item == addon.currenttab and addon.voting.container == addon.currentcontainer then
			-- local itemtext = select(2, GetItemInfo(itemID)) and select(2, GetItemInfo(itemID)) or addon.loots[itemID].link
			-- if addon.loots[itemID].quantity and addon.loots[itemID].quantity > 1 then
				-- itemtext = string.format("%s %sx", itemtext, addon.loots[itemID].quantity)
			-- end
			local itemtext = addon:GetItemText(containerID, itemID, "full")
			addon.window.item:SetText(itemtext .. L[", Voting closing in %d seconds"]:format(addon.opentime - timercount))
		end
		
		timer = self:ScheduleRepeatingTimer("UpdateVoteCloseTime", 1, {containerID, itemID})
		local numfinished, numcouncillors = 0, 0
		for _,councillor in pairs(addon.councillors) do
			if loot.votedone[councillor] then
				numfinished = numfinished + 1
			end
			numcouncillors = numcouncillors + 1
		end
		if numfinished == numcouncillors and addon.voting == itemID then
			addon:CloseVote(containerID, itemID)
		end
	else
		addon:CloseVote(containerID, itemID)
	end
end

function addon:CloseVote(containerID, itemID, from)
	if not from --[[ or addon.loots[itemID].starter == pname]] then
		addon:SendComm("closevote", containerID, itemID)
	end
	addon.voting = nil
	addon.opentime = nil
	addon.loots[containerID][itemID].starter = nil
	if timer then
		self:CancelTimer(timer, true)
	end
	addon:FinishVote(containerID, itemID)
	self:RedrawTab(containerID, itemID)
end

function addon:Vote(messagenum, containerID, itemID, value, from)
	local loot = addon.loots[containerID][itemID]
	if value then value = 1 else value = 0 end
	--Spew("vote", messagenum, itemID, value, from)
	if not from or from == pname then
		loot.messages[messagenum].votes[pname] = (value == 0) and nil or value
		addon:SendComm("vote", messagenum, containerID, itemID, value)
	else
		loot.messages[messagenum].votes[from] = (value == 0 and nil) or value
		self:RedrawTab(containerID, itemID)
	end
end

function addon:FinishVote(containerID, itemID, from)
	local loot = addon.loots[containerID][itemID]
	if not from or from == pname then
		if not loot.votedone[pname] then
			addon:SendComm("finishvote", containerID, itemID)
		end
		if (loot.stage == 1 or loot.stage == 2) and addon.window.button2 then
			addon.window.button2:SetDisabled(true)
		end
	end
	local name = from ~= nil and from or pname
	loot.votedone[name] = true
	local numfinished, numcouncillors = 0, 0
	for _,councillor in pairs(addon.councillors) do
		if loot.votedone[councillor] then
			numfinished = numfinished + 1
		end
		numcouncillors = numcouncillors + 1
	end
	--print(numfinished, #addon.councillors)
	if numfinished == numcouncillors and addon.voting and addon.voting.item == itemID and addon.voting.container == containerID then
		addon:CloseVote(containerID, itemID)
	end
	self:RedrawTab(containerID, itemID)
end

function addon:RedrawTab(containerID, itemID)
	if addon.window and addon.window:IsShown() and addon.window.tabs and addon.loots[containerID][itemID] and self.currentcontainer and self.currentcontainer == containerID and itemID == self.currenttab then
		addon.window.tabs:SelectTab(itemID)
	end
end

local last_raid_members = 0
function addon:RAID_ROSTER_UPDATE(event)
	-- print(event)
	local members = GetNumRaidMembers()
	if members < last_raid_members then
		-- print("lost member")
		
		for k, name in pairs(addon.councillors) do
			if UnitInRaid(name) == nil then
				addon:RemoveCouncillor(name)
			end
		end
		
		addon:RedrawTab(addon.currentcontainer, addon.currenttab)
	end
	last_raid_members = members
	-- if UnitInRaid("player") then
		-- if not addon.initraid then
			-- addon:Introduce()
			-- addon.initraid = true
		-- end
		
		-- local method, partyID, raidID = GetLootMethod()
		-- if method == "master" then
		
		-- else
			
		-- end
	-- else
		-- addon.initraid = false
		-- wipe(addon.councillors)
	-- end
end

function addon:CHAT_MSG_LOOT(event, msg)
	local player, item = df:Deformat(msg, LOOT_ITEM)
	if addon.lastloot then
		local lastloot = addon.loots[addon.lastloot.container][addon.lastloot.item]
		if not player then
			item = df:Deformat(msg, LOOT_ITEM_SELF)
			if item then
				player = UnitName("player")
			end
		end
		local quantity
		if item then
			quantity = tonumber(item:match("(%d-)$")) or 1
			itemID = tonumber(string.match(item, 'item:(%d+)'))
			if addon.loots[addon.lastloot.container][itemID] then
				local loot = addon.loots[addon.lastloot.container][itemID]
				if loot.looted[player] then
					loot.looted[player] = loot.looted[player] + quantity
				else
					loot.looted[player] = quantity
				end
				local count, amount = 0, loot.quantity
				for k,v in pairs(loot.looted) do
					count = count + 1
				end
				if count >= amount then
					loot.stage = 4
					if addon.voting and addon.voting.item == itemID and addon.voting.container == addon.lastloot.container then
						if timer then
							self:CancelTimer(timer, true)
						end
						_ = addon:CloseVote(addon.voting.container, addon.voting.item, pname, true)
					end
					if addon.open and addon.open.item == itemID and addon.open.container == addon.lastloot.container then
						if timer then
							self:CancelTimer(timer, true)
						end
						_ = addon:CloseLoot(addon.open.container, addon.open.item, pname, true)
					end
				end
				addon:RedrawTab(addon.lastloot.container, itemID)
			end
		end
	end
	-- player, item, quantity
end

function addon:ToggleWindow()
	if not self.window then
		self:CreateWindow()
	end
	if self.window:IsShown() then
		self.window:Hide()
	else
		self.window:Show()
	end
end
local guildMemberList = {}
addon.guildMemberList = guildMemberList
local coloredName
--local updating = nil
-- Helper table to cache hex colors for all classes.
local hexColors = setmetatable({}, {__index =
	function(self, key)
		if RAID_CLASS_COLORS[key] then
			local c = RAID_CLASS_COLORS[key]
			self[key] = "|cff" .. string.format("%02x%02x%02x", c.r * 255, c.g * 255, c.b * 255)
			return self[key]
		else
			return "|cffa0a0a0"
		end
	end
})

-- Helper table to cache colored player names.
local coloredName = setmetatable({}, {__index =
	function(self, key)
		if type(key) == "nil" then return nil end
		local class = select(2, UnitClass(key))
		if class then
			self[key] = hexColors[class] .. key .. "|r"
			return self[key]
		else
			return hexColors[key] .. key .. "|r"
		end
	end
})

function addon:TallyVotes(containerID, itemID)
	local loot = addon.loots[containerID][itemID]
	addon.lastloot = {container = containerID, item = itemID}
	-- QWER = loot
	-- Spew(itemID, loot)
	addon.rollers = {}
	for messagenum, message in pairs(loot.messages) do
		-- Spew(messagenum, message)
		local votes = 0
		for voter, vote in pairs(message.votes) do
			--Spew(voter, vote)
			if vote == 1 then
				votes = votes + 1
			end
		end
		-- Spew("votes", votes)
		if (votes / #addon.councillors) >= (settings.roll_percent /100) then
			-- print("Item Qualitifies for rolling")
			local inlist = false
			for _,v in pairs(addon.rollers) do
				if v == message.player then
					inlist = true
					-- break
				end
			end
			if not inlist then
				table.insert(addon.rollers, message.player)
			end
		end
	end
	-- MeSpew("rollers", addon.rollers)
	if #addon.rollers > loot.quantity then
		local newrollers = {}
		addon:Announce(table.concat(addon.rollers, ", ") .. L[" Roll out of %s now for %s (%s second timeout)"]:format(settings.roll_out_of, loot.link, settings.roll_time_out))
		for k,v in pairs(addon.rollers) do
			newrollers[v] = 0
		end
		addon.rollers = newrollers
		self:RegisterEvent("CHAT_MSG_SYSTEM", "RollHandler")
		for _,v in pairs(settings.channels) do
			if v:lower() == "raid" then
				addon:RegisterEvent("CHAT_MSG_RAID", "RollHandler")
				addon:RegisterEvent("CHAT_MSG_RAID_LEADER", "RollHandler")
			elseif v:lower() == "party" then
				addon:RegisterEvent("CHAT_MSG_PARTY", "RollHandler")
				addon:RegisterEvent("CHAT_MSG_PARTY_LEADER", "RollHandler")
			elseif v:lower() == "guild" then
				addon:RegisterEvent("CHAT_MSG_GUILD", "RollHandler")
			elseif v:lower() == "officer" then
				addon:RegisterEvent("CHAT_MSG_OFFICER", "RollHandler")
			else
				addon:RegisterEvent("CHAT_MSG_CHANNEL", "RollHandler")
				-- LootCouncil:RegisterEvent("CHAT_MSG_CHANNEL", "RollHandler")
			end
		end
		rolltimeout = self:ScheduleTimer("RollFinish", settings.roll_time_out)
		addon.rolling = {container = containerID, item = itemID}
	elseif #addon.rollers >= 1 and #addon.rollers <= loot.quantity then
		addon:Announce(L["%s wins %s"]:format(table.concat(addon.rollers, ", "), loot.link))
		-- MeSpew("tally votes: 1 winner", addon.rollers)
		addon:DistributeLoot(containerID, itemID, addon.rollers)
	end
end

function addon:RollHandler(event, message, sender, ...)
	-- MeSpew("RollerHandler", tostring(message))
	if sender ~= "" and sender ~= nil then
		local channelname = select(7, ...)
		-- Spew("channelname", channelname)
		local good = false
		for k,v in pairs(settings.channels) do
			if v:lower() == channelname:lower() then
				good = true
			end
		end
		if not good then
			return
		end
		if strtrim(string.lower(message)) == "pass" then
			-- print(("registered a pass from %s"):format(sender))
			addon.rollers[sender] = -1
		end
	end
	-- MeSpew("RollerHandler", {...})
	if addon.rolling == nil then return end
	local playername, roll, min, max = df(message, RANDOM_ROLL_RESULT)
	-- MeSpew(tostring(playername), {roll = tostring(roll), min = tostring(min), max = tostring(max)})
	-- print("roll", tostring(playername), tostring(roll), tostring(min), tostring(max))
	if playername and addon.rollers[playername] ~= nil then
		if min == 1 and max == settings.roll_out_of then
			if addon.rollers[playername] == 0 then
				addon.rollers[playername] = roll
			end
		end
	end
	local count = count(addon.rollers)
	local rolls = 0
	for k,v in pairs(addon.rollers) do
		-- MeSpew("v", v)
		if v ~= 0 then
			rolls = rolls +1
		end
	end
	-- MeSpew("rollers", addon.rollers)
	-- MeSpew("roller count", rolls)
	if count == rolls then
		addon:CancelTimer(rolltimeout, true)
		addon:RollFinish()
	end
end

function addon:RollFinish()
	self:UnregisterEvent("CHAT_MSG_SYSTEM")
	addon:AnnounceRollWinner(addon.rolling.container, addon.rolling.item)
	addon.rolling = nil
end

function addon:AnnounceRollWinner(containerID, itemID)
	local highroller = nil
	local loot = addon.loots[containerID][itemID]

	if loot.quantity > 1 then
		local winners = {}
		for k,v in pairs(addon.rollers) do
			table.insert(winners, {k, v})
		end
		table.sort(winners, function(a,b) return a[2] > b[2] end)
		if #winners > loot.quantity then
			for i=loot.quantity+1, #winners do
				winners[i] = nil
			end
		end
		-- MeSpew("rollers", winners)
		for k,v in pairs(winners) do
			winners[k] = v[1]
		end
		
		addon:Announce(L["%s wins %s"]:format(table.concat(winners, ", "), loot.link))
	else
		for k,v in pairs(addon.rollers) do
			if v > (addon.rollers[highroller] or -1) then
				highroller = k
			end
		end
		addon:Announce(L["%s wins %s"]:format(highroller, loot.link))
	end
	-- MeSpew("AnnounceRollWinner winners", (highroller ~= nil and highroller or winners))
	addon:DistributeLoot(containerID, itemID, (highroller ~= nil and {highroller} or winners))
end

function addon:DistributeLoot(containerID, itemID, winners, number)
	if type(containerID) == "table" then
		local containerID, itemID, winners, number = unpack(containerID)
	end
	local method, pid, rid = GetLootMethod()
	local mlid = addon:GetMLID(pname)
	if addon.can_auto_loot ~= containerID or not addon.can_auto_loot then return end
	-- print("can auto loot")
	if method ~= "master" then return end
	-- print("method is master")
	-- print(("self ID: %s, raid ID: %s, party ID: %s, my name: %s"):format(mlid, rid, pid, pname))
	-- print(("master looter: %s, my name: %s"):format(addon:GetMasterLooterName(), pname))
	-- if (rid ~= mlid or pid ~= mlid) then return end
	if (addon:GetMasterLooterName() ~= pname) then return end
	-- print("master looter is self")
	-- local itemName, itemLink, rarity = GetItemInfo(itemID)
	local itemName, itemLink, itemQuality, _, _, _, _, _, _, itemTexture = GetItemInfo(itemID)
	local r,g,b = GetItemQualityColor(itemQuality)
	local loot = addon.loots[containerID][itemID]
	local first = nil
	MeSpew("winners", winners)
	if #winners > 1 then
		if number ~= nil then
			first = winners[number][1]
		else
			number = 1
			first = winners[1][1]
		end
	else
		first = winners[1]
	end
	-- print(tostring(itemID), tostring(number), tostring(first))
	StaticPopup_Hide("CONFIRM_LOOT_DISTRIBUTION_LC")
	local name = coloredName[first] ~= nil and coloredName[first] or first
	-- MeSpew("name", first)
	-- MeSpew("coloredName[first]", coloredName[first])
	dialog = StaticPopup_Show("CONFIRM_LOOT_DISTRIBUTION_LC", ITEM_QUALITY_COLORS[itemQuality].hex..itemName..FONT_COLOR_CODE_CLOSE, name, {["texture"] = itemTexture, ["name"] = itemName, ["color"] = {r, g, b, 1}, ["link"] = itemLink, ["count"] = 1})
	addon.lastloot = {container = containerID, item = itemID}
	if dialog then
		dialog.data = {id = addon:GetMLID(first), slot = addon.loot_cache[loot.link]}
		if #winners > 1 then
			if number ~= nil then
				if (number +1) <= #winners then
					dialog.data["number"] = number +1
					dialog.data["winners"] = winners
					dialog.data["itemID"] = itemID
					dialog.data["containerID"] = containerID
				end
			else
				dialog.data["number"] = 1
				dialog.data["winners"] = winners
				dialog.data["itemID"] = itemID
				dialog.data["containerID"] = containerID
			end
		end
	end	
end

-- QWER = coloredName

function addon:GetMLID(name)
	if GetNumRaidMembers() > 0 then
		for i = 1, 40 do
			if GetMasterLootCandidate(i) == name then
				return i
			end
		end
	elseif GetNumPartyMembers() > 0 then
		for i = 1, MAX_PARTY_MEMBERS+1 do
			if GetMasterLootCandidate(i) == name then
				return i
			end
		end
	end
	return nil
end

function addon:GetMasterLooterName()
	if GetNumRaidMembers() > 0 then
		for i = 1, 40 do
			name, _, _, _, _, _, _, _, _, _, isML = GetRaidRosterInfo(i)
			if isML == 1 then
				return name
			end
		end
	elseif GetNumPartyMembers() > 0 then
		for i = 1, MAX_PARTY_MEMBERS+1 do
			name, _, _, _, _, _, _, _, _, _, isML = GetRaidRosterInfo(i)
			if isML == 1 then
				return name
			end
		end
	end
	return nil
end

function addon:PercentColor(percent)
	local r,g,b
	if percent <= 0 then
		r,g,b = 1, 0, 0
	elseif percent <= 0.5 then
		r,g,b = 1, percent*2, 0
	elseif percent >= 1 then
		r,g,b = 0, 1, 0
	else
		r,g,b = 2 - percent*2, 1, 0
	end
	return string.format("%02x%02x%02x", r*255, g*255, b*255)
end

local function AltsMains(attende, player)
	if attende == player then
		return true
	end
	if addon.alts[player] ~= nil then
		for _,alt in pairs(addon.alts[player]) do
			if alt == attende then
				return true
			end
		end
	end
	for main,alts in pairs(addon.alts) do
		for _,alt in pairs(alts) do
			if alt == player and main == attende then
				return true
			end
		end
	end
	return false
end

local function copyToTable(src, dst)
	wipe(dst)
	for i, v in pairs(src) do dst[i] = v end
end

local function GetAltsMains(name)
	local main = name
	local alts = {}
	if addon.alts[name] ~= nil then
		main = name
		copyToTable(addon.alts[name], alts)
	else
		for m,a in pairs(addon.alts) do
			for _,aa in pairs(a) do
				if aa == name then
					main = m
					copyToTable(a, alts)
				end
			end
		end
	end
	local str = coloredName[main]
	if type(alts) == "table" and #alts > 0 then
		for n,a in pairs(alts) do
			alts[n] = coloredName[a]
		end
		str = str .. L[" (%s)"]:format(table.concat(alts, ", "))
	end
	return "|cffffffff" .. str .. "|r"
end

function addon:PlayerTooltipOnEnter(...)
	-- print("PlayerTooltipOnEnter")
	-- MeSpew("...", self:GetName())
	addon:PlayerTooltip(self, _G[self:GetName() .. "Name"]:GetText())
end
function addon:PlayerTooltipOnLeave(...)
	-- print("PlayerTooltipOnEnter")
	-- MeSpew("...", self:GetName())
	GameTooltip:Hide()
end

function addon:PlayerTooltip(frame, player)
	if NRT and NRT.db.profile then
		local attendance, loot = NRT.db.profile.tracked, NRT.db.profile.loot
		if frame ~= nil then
			GameTooltip:SetOwner(frame, 'ANCHOR_NONE')
			local x,y = frame:GetCenter()
			local vhalf = (y > UIParent:GetHeight()/2) and "TOP" or "BOTTOM"
			local hhalf = (x < UIParent:GetWidth()/2) and "LEFT" or "RIGHT"
			GameTooltip:SetPoint(vhalf..hhalf, frame, (hhalf == "LEFT" and "RIGHT" or "LEFT"), (hhalf == "LEFT" and 10 or -10), 0)
		else
			GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
		end
		-- GameTooltip:AddLine(coloredName[player])
		GameTooltip:AddLine(GetAltsMains(player))
		GameTooltip:AddLine(" ")
		GameTooltip:AddLine(L["|cffffffffAttendance|r"])
		local raids = {}
		local week = 604800
		local thetime = time()
		local week2_count, week2, week4_count, week4, week6_count, week6, shown = 0,0,0,0,0,0,0
		local countinraid = 0
		for i=#attendance, 1, -1 do
			local inraid = false
			for _,name in pairs(attendance[i]["attendants"]) do
				-- if name == player then
				if AltsMains(name,player) then
					if countinraid < 5 and (#attendance - i) < 10 then
						table.insert(raids, L["|cff00ff00(%s) %s - %s|r"]:format(date("%m/%d %H:%M", attendance[i]["date"]), attendance[i]["zone"], attendance[i]["note"] or attendance[i]["name"]))
						shown = shown + 1
					end
					if attendance[i]["date"] >= (thetime - (week*2)) then
						week2 = week2 + 1
					end
					if attendance[i]["date"] >= (thetime - (week*4)) then
						week4 = week4 + 1
					end
					if attendance[i]["date"] >= (thetime - (week*6)) then
						week6 = week6 + 1
					end
					inraid = true
					countinraid = countinraid + 1
				end
			end
			if not inraid then
				if countinraid < 5 and (#attendance - i) < 10 then
					table.insert(raids, L["|cffff0000(%s) %s - %s|r"]:format(date("%m/%d %H:%M", attendance[i]["date"]), attendance[i]["zone"], attendance[i]["note"] or attendance[i]["name"]))
				end
			end
			if attendance[i]["date"] >= (thetime - (week*2)) then
				week2_count = week2_count + 1
			end
			if attendance[i]["date"] >= (thetime - (week*4)) then
				week4_count = week4_count + 1
			end
			if attendance[i]["date"] >= (thetime - (week*6)) then
				week6_count = week6_count + 1
			end
		end
		GameTooltip:AddLine(L["  |cffffffffLast 2 Weeks:|r |cff%s%.2f%%|r"]:format(addon:PercentColor(week2/week2_count > 0 and week2/week2_count or 0), (week2/week2_count)*100 > 0 and (week2/week2_count)*100 or 0))
		GameTooltip:AddLine(L["  |cffffffffLast 4 Weeks:|r |cff%s%.2f%%|r"]:format(addon:PercentColor(week4/week4_count > 0 and week4/week4_count or 0), (week4/week4_count)*100 > 0 and (week4/week4_count)*100 or 0))
		GameTooltip:AddLine(L["  |cffffffffLast 6 Weeks:|r |cff%s%.2f%%|r"]:format(addon:PercentColor(week6/week6_count > 0 and week6/week6_count or 0), (week6/week6_count)*100 > 0 and (week6/week6_count)*100 or 0))
		GameTooltip:AddLine(L["  |cffffffffLifetime:|r |cff%s%.2f%%|r"]:format(addon:PercentColor(countinraid/#(attendance) > 0 and countinraid/#(attendance) or 0), (countinraid/#(attendance))*100 > 0 and (countinraid/#(attendance))*100 or 0))
		for _,v in pairs(raids) do
			GameTooltip:AddLine(v)
		end
		GameTooltip:AddLine(L["%d Total (%d shown) Recorded Attendance Records"]:format(countinraid, shown))
		GameTooltip:AddLine(" ")
		GameTooltip:AddLine(L["|cffffffffLoot|r"])
		local loots = {}
		local countloots = 0
		for i=#loot, 1, -1 do
			if AltsMains(loot[i]["player"],player) then
				if countloots <= 5 then
					table.insert(loots, L["(%s) %s - %s"]:format(date("%m/%d %H:%M", loot[i]["date"]), loot[i]["from"], loot[i]["item"]))
				end
				countloots = countloots + 1
			end
		end
		for _,v in pairs(loots) do
			GameTooltip:AddLine(v)
		end
		if (countloots - 5) > 0 then
			GameTooltip:AddLine(L["%d previous items"]:format(countloots - 5))
		end
		GameTooltip:Show()
	else
		GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
		GameTooltip:AddLine(L["Mihilim Raid Tracker (NRT) Not Found"])
		GameTooltip:AddLine(L["http://www.wowace.com/addons/nrt/"])
		GameTooltip:Show()
	end
end
addon.alts = {}
do 
	local updating = nil
	function addon:GUILD_ROSTER_UPDATE()
		addon:UnregisterEvent("GUILD_ROSTER_UPDATE")
		if updating then return end
		updating = true
		local numGuildMembers = GetNumGuildMembers(true)
		for i = 1, numGuildMembers do
			local name, _, _, level, _, _, note, onote, _, _, class = GetGuildRosterInfo(i)
			if name then
				if rawget(coloredName, name) then
					coloredName[name] = nil
				end
				if level >= 80 then
					guildMemberList[name] = {n = note, on = onote}
				end
			end
		end
		for name, v in pairs(guildMemberList) do
			-- local note = strsplit(" '
			for str in string.gmatch(v.n, "([^%c%p%s%z%d]+)") do
				-- print(str)
				if guildMemberList[str] ~= nil then
					if addon.alts[str] == nil then addon.alts[str] = {} end
					insertUnique(addon.alts[str], name)
				end
			end
			for str in string.gmatch(v.on, "([^%c%p%s%z%d]+)") do
				-- print(str)
				if guildMemberList[str] ~= nil then
					if addon.alts[str] == nil then addon.alts[str] = {} end
					insertUnique(addon.alts[str], name)
				end
			end
		end
		-- Spew("alts", addon.alts)
		updating = nil
	end
	
	local window = nil
	local raidselect = nil
	local AceGUI = LibStub("AceGUI-3.0")
	AceGUI:RegisterLayout("LootCouncil_List",
	 function(content, children)
		local height = 0
	 	local width = content.width or content:GetWidth() or 0
		local numchildren = #children
		for i, child in ipairs(children) do
			local frame = child.frame
			frame:ClearAllPoints()
			frame:Show()
			if i == 1 then
				frame:SetPoint("LEFT",content,"LEFT",0,0)
				frame:SetWidth(100)
			elseif i == 2 then
				frame:SetPoint("LEFT",children[1].frame,"RIGHT",0,0)
				child:SetWidth( width - 100 - 40 - (numchildren-3)*22 )
				--child.label:SetWidth( width - 100 - (numchildren-2)*20 )
			elseif i == #children then
				frame:SetPoint("RIGHT",content,"RIGHT",0,0)
			else
				frame:SetPoint("RIGHT",children[i+1].frame,"LEFT",0,0)
				--child:SetDisabled(true)
			end
			
			if i == 3 then
				if children[4] and children[4].frame then
					frame:SetPoint("RIGHT",children[4].frame,"LEFT",0,0)
				end
				child:SetWidth( 40 )
			end
			
			--[[if child.width == "fill" then
				child:SetWidth(width)
				frame:SetPoint("RIGHT",content,"RIGHT")
				if child.OnWidthSet then
					child:OnWidthSet(content.width or content:GetWidth())
				end
				if child.DoLayout then
					child:DoLayout()
				end
			elseif child.width == "relative" then
				child:SetWidth(width * child.relWidth)
				if child.OnWidthSet then
					child:OnWidthSet(content.width or content:GetWidth())
				end
				if child.DoLayout then
					child:DoLayout()
				end
			end]]
			
			height = height + (frame.height or frame:GetHeight() or 0)
		end
		--safecall( content.obj.LayoutFinished, content.obj, nil, height )
	 end
	)
	local emptytable = {}
	function addon:CreateWindow()
		
		
		local window = AceGUI:Create("Window")
		--f:SetCallback("OnClose",function(widget, event) AceGUI:Release(widget) end )
		window:SetTitle(L["Loot Council"] .. " " .. windowString)
		--f:SetStatusText("Raid info status")
		window:SetLayout("Flow")
		window.frame:SetMinResize((300+ #addon.councillors *22+100),300)
		
		local clearloots = AceGUI:Create("Button")
		clearloots:SetRelativeWidth(0.50)
		clearloots:SetText(L["Clear Loots"])
		clearloots:SetCallback("OnClick", function()
			StaticPopup_Show("LC_CLEARLOOTS")
		end)
		window:AddChild(clearloots)
		
		local gostandby = AceGUI:Create("Button")
		gostandby:SetRelativeWidth(0.50)
		gostandby:SetText(L["Go Standby"])
		
		gostandby:SetCallback("OnClick", function()
			if is_standby then
				is_standby = false
				addon:ReturnStandby()
				gostandby:SetText(L["Go Standby"])
			else
				is_standby = true
				addon:GoStandby()
				gostandby:SetText(L["Come Back"])
				
			end
		end)
		window:AddChild(gostandby)
		
		-- local openattendance = AceGUI:Create("Button")
		-- openattendance:SetRelativeWidth(0.20)
		-- openattendance:SetText(L["Open Attendance"])
		-- window:AddChild(openattendance)
		
		-- local takeattendance = AceGUI:Create("Button")
		-- takeattendance:SetRelativeWidth(0.20)
		-- takeattendance:SetText(L["Take Attendance"])
		-- window:AddChild(takeattendance)
		
		-- local dovote = AceGUI:Create("Button")
		-- dovote:SetRelativeWidth(0.20)
		-- dovote:SetText(L["Vote on Anything"])
		-- window:AddChild(dovote)
		
		local councillors = AceGUI:Create("Label")
		--councillors:SetFontObject("GameFontNormal")
		local str = {}
		for _,v in pairs(addon.councillors) do
			local text
			if addon.versions[v] ~= nil then
				if v ~= pname then
					if addon.versions[v] == -1 or addon.versions[v] == addon.versions[pname] then
						text = v .. " (" .. addon.versions[v] .. ")"
					elseif addon.versions[v] > addon.versions[pname] then
						text = "|cff00ff00" .. v .. " (" .. addon.versions[v] .. ")|r"
					else
						text = "|cffff0000" .. v .. " (" .. addon.versions[v] .. ")|r"
					end
				else
					text = v .. " (" .. addon.versions[v] .. ")"
				end
			else
				text = "|cffff0000" .. v .. " (??)|r"
			end
			
			table.insert(str, text)
		end
		local strstr = table.concat(str, ", ")
		councillors:SetText(L["Councillors: %s"]:format(strstr ~= "" and strstr or L["None"]) )
		window:AddChild(councillors)
		window.councillors = councillors
		councillors:SetFullWidth(true)
		local function DrawLoot(container, group)
			local loot = addon.loots[addon.currentcontainer][tonumber(group)]
			if loot.stage == 0 then
				local button = AceGUI:Create("Button")
				button:SetFullWidth(true)
				button:SetText(L["Open Item for Council"])
				
				button:SetCallback("OnClick", function()
					addon:StartLoot(addon.currentcontainer, tonumber(group))
				end)
				
				container:AddChild(button)
				window.button1 = button
				window.button2 = nil
			end
			
			if loot.stage == 1 then
				local button = AceGUI:Create("Button")
				button:SetRelativeWidth(0.5)
				button:SetText(L["Start Vote"])
				if loot.starter ~= pname then
					button:SetDisabled(true)
				else
					button:SetDisabled(false)
				end
				button:SetCallback("OnClick", function()
					addon:StartVote(addon.currentcontainer, tonumber(group), settings.vote_time)
				end)
				container:AddChild(button)
				window.button1 = button
				local button = AceGUI:Create("Button")
				button:SetRelativeWidth(0.5)
				button:SetText(L["Finished Voting"])
				-- if loot.votedone[pname] then
					button:SetDisabled(true)
				-- else
					-- button:SetDisabled(false)
				-- end
				button:SetCallback("OnClick", function()
					addon:FinishVote(addon.currentcontainer, group)
				end)
				container:AddChild(button)
				window.button2 = button
			end
			
			if loot.stage == 2 then
				local button = AceGUI:Create("Button")
				button:SetRelativeWidth(0.5)
				button:SetText(L["Tally Votes"])
				
				button:SetCallback("OnClick", function()
					addon:TallyVotes(addon.currentcontainer, tonumber(group))
				end)
				container:AddChild(button)
				window.button1 = button
				local button = AceGUI:Create("Button")
				button:SetRelativeWidth(0.5)
				if loot.votedone[pname] then
					button:SetDisabled(true)
				else
					button:SetDisabled(false)
				end
				button:SetText(L["Finished Voting"])
				button:SetCallback("OnClick", function()
					addon:FinishVote(addon.currentcontainer, tonumber(group))
				end)
				container:AddChild(button)
				window.button2 = button
			end
			
			local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, _, itemEquipLoc, itemTexture = GetItemInfo(tonumber(group))
			local item = AceGUI:Create("InteractiveLabel")
			local currentitemlink = itemLink and itemLink or loot.link
			item:SetFullWidth(true)
			item:SetFont(GameFontNormal:GetFont(), 20)
			-- local text = (itemLink ~= nil) and itemLink or loot.link
			local text = addon:GetItemText(addon.currentcontainer, currentitemlink, "full")
			--Spew("looted", addon.loots[group].looted)
			local lootedcount = 0
			for k in pairs(loot.looted) do
				lootedcount = lootedcount + 1
			end
			--print("#loot.looted", #loot.looted)
			if lootedcount >= 1 then
				local c, cc = lootedcount, 1
				--print("c", c)
				
				for k in pairs(loot.looted) do
					if cc == 1 then
						text = text .. L[", Looted by: "]
					end
					text = text .. coloredName[k]
					--print("cc", cc)
					if c > cc then
						text = text .. ", "
					end
					cc = cc + 1
				end
			end
			if addon.open and addon.open.container == addon.currentcontainer and addon.open.item == group then
				item:SetText(text .. L[", Closing in %d seconds"]:format(addon.opentime - timercount))
			elseif addon.voting and addon.voting.container == addon.currentcontainer and addon.voting.item == group then
				item:SetText(text .. L[", Voting closing in %d seconds"]:format(addon.opentime - timercount))
			else
				item:SetText(text)
			end
			item:SetHighlight("Interface\\QuestFrame\\UI-QuestTitleHighlight")
			item:SetImage(itemTexture)
			item:SetImageSize(30,30)
			item:SetUserData("item", group)
			
			item:SetCallback("OnEnter", function()
				GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
				GameTooltip:SetHyperlink(itemLink and itemLink or loot.link)
				GameTooltip:Show()
				ShoppingTooltip1:Hide()
				ShoppingTooltip2:Hide()
			end)
			item:SetCallback("OnLeave", function()
				GameTooltip:Hide()
			end)
			window.item = item
			container:AddChild(item)
			
			
			
			local headings = AceGUI:Create("SimpleGroup")
			headings:SetFullWidth(true)
			headings:SetHeight(20)
			headings:SetLayout("LootCouncil_List")
			
			local playername = AceGUI:Create("Label")
			playername:SetFont(GameFontNormal:GetFont(),13)
			playername:SetText(L["Player"])
			headings:AddChild(playername)
			
			local message = AceGUI:Create("Label")
			message:SetFont(GameFontNormal:GetFont(),13)
			message:SetText(L["Item Message"])
			headings:AddChild(message)
			
			local msg_itemlevel = AceGUI:Create("Label")
			msg_itemlevel:SetFont(GameFontNormal:GetFont(),13)
			msg_itemlevel:SetText(L["iLevel"])
			headings:AddChild(msg_itemlevel)
			
			for _,councillor in pairs(addon:GetCouncillors()) do
				if (pname == councillor and settings.private == "true") or settings.private == "false" then
					local councilicon = AceGUI:Create("Icon")
					councilicon:SetWidth(22)
					--councilicon:SetDisabled(false)
					councilicon:SetImageSize(18,18)
					if loot.votedone[councillor] then
						councilicon:SetImage("Interface\\RAIDFRAME\\ReadyCheck-Ready")
					else
						councilicon:SetImage("Interface\\RAIDFRAME\\ReadyCheck-NotReady")
					end
					--councilicon:SetDisabled(false)
					councilicon:SetCallback("OnEnter", function()
						GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
						GameTooltip:AddLine(councillor)
						GameTooltip:Show()
					end)
					councilicon:SetCallback("OnLeave", function()
						GameTooltip:Hide()
					end)
					
					councilicon:SetDisabled(false)
					councilicon.image:SetAlpha(1)
					window.votedone = window.votedone or {}
					window["votedone"][councillor] = councilicon
					headings:AddChild(councilicon)
				end
			end
			
			container:AddChild(headings)
			
			local scroll = AceGUI:Create("ScrollFrame")
			scroll:SetFullHeight(true)
			scroll:SetFullWidth(true)
			scroll:SetLayout("List")
			container:AddChild(scroll)
			--local players = {}
			
			for messagenum,message in pairs(loot.messages or emptytable) do
				--local number = #players+1
				--for messagenum,message in pairs(messages) do
					local pgroup = AceGUI:Create("SimpleGroup")
					pgroup:SetFullWidth(true)
					pgroup:SetHeight(20)
					pgroup:SetLayout("LootCouncil_List")
					
					local playername = AceGUI:Create("InteractiveLabel")
					--playername:SetWidth(100)
					playername:SetFont(GameFontNormal:GetFont(),13)
					--playername:SetRelativeWidth(0.2)
					playername:SetText(coloredName[message.player])
					playername:SetHighlight("Interface\\QuestFrame\\UI-QuestTitleHighlight")
					playername:SetCallback("OnEnter", function()
						addon:PlayerTooltip(self, message.player)
						-- GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
						-- GameTooltip:AddLine(coloredName[message.player])
						-- GameTooltip:AddLine(L["Attendance"])
						
						-- GameTooltip:AddLine(L["Loot"])
						
						
						-- GameTooltip:SetHyperlink(itemLink)
						
						-- GameTooltip:Show()
					end)
					playername:SetCallback("OnLeave", function()
						GameTooltip:Hide()
					end)
					pgroup:AddChild(playername)
					
					local pitem = AceGUI:Create("InteractiveLabel")
					local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, _, itemEquipLoc, itemTexture = GetItemInfo(message.message)
					--pitem:SetRelativeWidth(0.8)
					--pitem:SetFullWidth(true)
					--pitem:SetFontObject(GameFontNormal)
					pitem:SetFont(GameFontNormal:GetFont(),13)
					pitem:SetText(message.message)
					pitem:SetHighlight("Interface\\QuestFrame\\UI-QuestTitleHighlight")
					--pitem:SetImage(itemTexture)
					--pitem:SetImageSize(14,14)
					
					pitem:SetCallback("OnEnter", function()
						GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
						-- Spew("itemlink", itemLink)
						GameTooltip:SetHyperlink(itemLink or message.message)
						ShoppingTooltip1:SetHyperlink(currentitemlink)
						ShoppingTooltip1:Show()
						ShoppingTooltip2:Hide()

						--GameTooltip_SetDefaultAnchor( ItemRefTooltip, UIParent )
						--ItemRefTooltip:SetHyperlink(item.link)
						GameTooltip:Show()
					end)
					pitem:SetCallback("OnLeave", function()
						GameTooltip:Hide()
						ShoppingTooltip1:Hide()
						ShoppingTooltip2:Hide()
					end)
					pgroup:AddChild(pitem)
					
					local itemlevel = AceGUI:Create("Label")
					itemlevel:SetFont(GameFontNormal:GetFont(),13)
					itemlevel:SetText(itemLevel ~= nil and itemLevel or L["??"])
					pgroup:AddChild(itemlevel)
					
					for _,councillor in pairs(addon:GetCouncillors()) do
						if pname == councillor then
							local councilcheck = AceGUI:Create("CheckBox")
							councilcheck:SetWidth(22)
							councilcheck:SetType("checkbox")
							if not loot.votedone[councillor] then
								councilcheck:SetDisabled(false)
							else
								councilcheck:SetDisabled(true)
							end
							councilcheck:SetValue(message.votes[councillor] == 1 and true or false)
							councilcheck:SetCallback("OnValueChanged", function(widget, event, value)
								addon:Vote(messagenum, addon.currentcontainer, group, value)
							end)
							pgroup:AddChild(councilcheck)
						else
							if settings.private == "false" then
								local councilicon = AceGUI:Create("Icon")
								councilicon:SetWidth(22)
								councilicon:SetImageSize(18,18)
								if loot.votedone[councillor] then
									councilicon:SetDisabled(false)
									councilicon.image:SetAlpha(1)
								else
									councilicon:SetDisabled(true)
								end
								if message.votes[councillor] == 1 then
									councilicon:SetImage("Interface\\RAIDFRAME\\ReadyCheck-Ready")
								else
									councilicon:SetImage("Interface\\RAIDFRAME\\ReadyCheck-NotReady")
								end
								councilicon:SetCallback("OnEnter", function()
									GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
									GameTooltip:AddLine(councillor)
									GameTooltip:Show()
								end)
								councilicon:SetCallback("OnLeave", function()
									GameTooltip:Hide()
								end)
								
								pgroup:AddChild(councilicon)
							end
						end
					end
					scroll:AddChild(pgroup)
				--end
			end
			addon.currentscroll = scroll
		end
		-- Callback function for OnGroupSelected
		local function SelectGroup(container, event, group)
			container:ReleaseChildren()
			addon.currenttab = tonumber(group)
			DrawLoot(container, group)
		end
		local function SelectMob(container, event, guid)
			container:ReleaseChildren()
			addon.currentcontainer = guid
			
			local loottabs = AceGUI:Create("TabGroup")
			loottabs:SetLayout("Flow")
			-- Setup which tabs to show
			loottabs:SetTabs(addon:GetTabs(guid))
			-- Register callback
			loottabs:SetCallback("OnGroupSelected", SelectGroup)
			-- Set initial Tab (this will fire the OnGroupSelected callback)
			local tab = addon:GetTabs(guid, true)
			if addon.open and addon.open.container == addon.currentcontainer then
				tab = addon.open.item
			elseif addon.voting and addon.voting.container == addon.currentcontainer then
				tab = addon.voting.item
			end
			-- MeSpew("SelectMob:tab", tostring(tab))
			loottabs:SelectTab(tab)
			loottabs:SetFullWidth(true)
			loottabs:SetFullHeight(true)
			if addon.window then
				addon.window.tabs = loottabs
			end
			container:AddChild(loottabs)
			
		end
		
		local mobtabs = AceGUI:Create("TabGroup")
		mobtabs:SetLayout("Flow")
		mobtabs:SetCallback("OnGroupSelected", SelectMob)
		mobtabs:SetTabs(addon:GetMobs())
		mobtabs:SelectTab(addon:GetMobs(true))
		mobtabs:SetFullWidth(true)
		mobtabs:SetFullHeight(true)
		window:AddChild(mobtabs)
		
		window:Hide()
		_G['LootCouncilFrame'] = window
		tinsert(UISpecialFrames,"LootCouncilFrame")
		self.window = window
		self.window.mobtabs = mobtabs
		
		--tinsert(UISpecialFrames,"LootCouncilWindow")
		--ASDF = window
		
	end
end

